#include "artecategorycatalog.h"
#include <preferences.h>
#include <film/filmdetails.h>
#include "artedefinitions.h"

#include <QDebug>

ArteCategoryCatalog::ArteCategoryCatalog(QObject* parent)
    :QObject(parent)
{
    // Expected Language Code : fr/de
    QString languageCode = Preferences::getInstance()->applicationLanguage().toLower();

    QMap<QString, QString> map;
    // Infos et société | Politics and society | Aktuelles und Gesellschaft
    map[tr("Politics and society")] = "ACT";

    // Cinéma | ?? | Kino
    map[tr("Movies")] = "CIN";

    // Séries et fiction | ?? | Fernsehfilme und Serien
    map[tr("TV Series")] = "SER";

    // Culture et pop | Culture | Kultur und Pop
    map[tr("Culture")] = "CPO";

    // Arte Concert | Arte Concert | Arte Concert
    map[tr("Arte Concert")] = "ARS";

    // Sciences | Sciences | Wissenschaft
    map[tr("Sciences")] = "SCI";

    // Voyages et découvertes | Discovery | Entdeckung der Welt
    map[tr("Discovery")] = "DEC";

    // Histoire | History | Geschichte
    map[tr("History")] = "HIS";

    QMapIterator<QString, QString> i(map);

    while (i.hasNext()) {
        i.next();
        m_urlByCatalogName.insert(i.key(), QString("http://www.arte.tv/hbbtvv2/services/web/index.php/EMAC/teasers/category/v2/%0/%1")
                                  .arg(i.value())
                                  .arg(languageCode));
    }
}

QList<FilmDetails*> ArteCategoryCatalog::listFilmsFromCatalogAnswer(QString catalogName, const QString& catalogAnswer){

    QList<FilmDetails*> resultList;

    QList<QVariant> list = extractJsonMapFromAnswer(catalogAnswer).value("category").toList();

    foreach(QVariant subcategory, list) {
        foreach(QVariant catalogItemVariant, subcategory.toMap().value("teasers").toList()) {
            auto catalogItem = catalogItemVariant.toMap();

            QStringList unsupportedKinds = {"TOPIC", "MAGAZINE", "TV_SERIES"};
            if (unsupportedKinds.contains(catalogItem.value("kind").toString())) {
                // These kinds are not supported for now: they often contains a list of items.
                continue;
            }

            FilmDetails* newFilm = new FilmDetails(catalogName,
                                                   catalogItem.value("title").toString(),
                                                   catalogItem.value("url").toString(),
                                                   catalogItem.value("programId").toString());

            addMetadataIfNotEmpty(newFilm, catalogItem, "kind", Kind);
            if (!addMetadataIfNotEmpty(newFilm, catalogItem, "shortDescription", Description))
            {
                addMetadataIfNotEmpty(newFilm, catalogItem, "teaserText", Description);
            }
             extractMainArteMetaFromJsonMap(newFilm, catalogItem);

            if (catalogItem.value("durationSeconds").toString().toInt() > 0) {
                // Used by arte date catalog
                newFilm->m_durationInMinutes = catalogItem.value("durationSeconds").toString().toInt() / 60;
            } else {
                // Used by most recent catalog
                newFilm->m_durationInMinutes = catalogItem.value("duration").toInt();
            }

            QString imageUrl = catalogItem.value("imageUrl").toString();
            if (!imageUrl.isEmpty()){
                emit requestImageDownload(newFilm, imageUrl);
            }

            resultList.append(newFilm);
        }
    }

    return resultList;
}
